module Sheriff

  class FeatureSet

    attr_reader :name
    attr_accessor :context

    def initialize(name)
      @name = name
      @jurisdiction = Sheriff::Jurisdiction.new()
    end

    def hide(*args)
      @jurisdiction.hide *args
    end

    def show(*args)
      @jurisdiction.show *args
    end

    def visible?
      caller.first.match /`(.+)'$/
      context_method = $1
      @jurisdiction.can? context_method.to_sym, @context
    end

    def self.define(feature_set_name)
      if self.defined? feature_set_name
        raise Sheriff::FeatureSet::AlreadyDefinedError.new("Feature Set #{feature_set_name} already defined")
      end

      new_feature_set = Sheriff::FeatureSet.new(feature_set_name)
      Sheriff.defined_feature_sets << new_feature_set
      yield(new_feature_set) if block_given?
    end
    
    def self.defined?(feature_set_name)
      Sheriff.defined_feature_sets.map(&:name).include?(feature_set_name)
    end

    def self.find(name)
      Sheriff.defined_feature_sets.select {|fs| fs.name == name}.first
    end

    def self.all
      Sheriff.defined_feature_sets
    end
  end

end
